@@ -61,7 +61,7 @@ class PartialCredentialsError(BotoCoreError):
     :ivar cred_var: The missing credential variable name.
 
     """
-    fmt = 'Partial credentials found, missing: {cred_var}'
+    fmt = 'Partial credentials found in {provider}, missing: {cred_var}'
 
 
 class NoRegionError(BotoCoreError):
@@ -71,7 +71,9 @@ class NoRegionError(BotoCoreError):
     :ivar env_var: The name of the environment variable to use to
         specify the default region.
     """
-    fmt = 'You must specify a region or set the {env_var} environment variable.'
+    fmt = (
+        'You must specify a region or set the {env_var} environment variable.'
+    )
 
 
 class UnknownSignatureVersionError(BotoCoreError):
@@ -93,6 +95,18 @@ class ServiceNotInRegionError(BotoCoreError):
     fmt = 'Service {service_name} not available in region {region_name}'
 
 
+class UnknownEndpointError(BotoCoreError):
+    """
+    Could not construct an endpoint.
+
+    :ivar service_name: The name of the service.
+    :ivar region_name: The name of the region.
+    """
+    fmt = (
+        'Unable to construct an endpoint for '
+        '{service_name} in region {region_name}')
+
+
 class ProfileNotFound(BotoCoreError):
     """
     The specified configuration profile was not found in the
@@ -151,10 +165,13 @@ class ValidationError(BotoCoreError):
            "of type {type_name} ")
 
 
+class ParamValidationError(BotoCoreError):
+    fmt = 'Parameter validation failed:\n{report}'
+
+
 # These exceptions subclass from ValidationError so that code
 # can just 'except ValidationError' to catch any possibly validation
 # error.
-
 class UnknownKeyError(ValidationError):
     """
     Unknown key in a struct paramster.
@@ -188,8 +205,10 @@ class UnknownParameterError(ValidationError):
     :ivar operation: The name of the operation.
     :ivar choices: The valid choices the parameter name can be.
     """
-    fmt = ("Unknown parameter '{name}' for operation {operation}.  Must be one "
-           "of: {choices}")
+    fmt = (
+        "Unknown parameter '{name}' for operation {operation}.  Must be one "
+        "of: {choices}"
+    )
 
 
 class UnknownServiceStyle(BotoCoreError):
@@ -205,6 +224,10 @@ class PaginationError(BotoCoreError):
     fmt = 'Error during pagination: {message}'
 
 
+class OperationNotPageableError(BotoCoreError):
+    fmt = 'Operation cannot be paginated: {operation_name}'
+
+
 class EventNotFound(BotoCoreError):
     """
     The specified event name is unknown to the system.
@@ -250,3 +273,22 @@ class InvalidExpressionError(BotoCoreError):
 class UnknownCredentialError(BotoCoreError):
     """Tried to insert before/after an unregistered credential type."""
     fmt = 'Credential named {name} not found.'
+
+
+class WaiterConfigError(BotoCoreError):
+    """Error when processing waiter configuration."""
+    fmt = 'Error processing waiter config: {error_msg}'
+
+
+class ClientError(Exception):
+    MSG_TEMPLATE = (
+        'An error occurred ({error_code}) when calling the {operation_name} '
+        'operation: {error_message}')
+
+    def __init__(self, error_response, operation_name):
+        msg = self.MSG_TEMPLATE.format(
+            error_code=error_response['Error']['Code'],
+            error_message=error_response['Error']['Message'],
+            operation_name=operation_name)
+        super(ClientError, self).__init__(msg)
+        self.response = error_response
